import React, { useState, forwardRef, useRef, useEffect, useMemo } from 'react';
import { useClickOutside } from 'react-click-outside-hook';
import noop$1 from 'lodash/noop';
import { useAutoPositioner, useForkRef } from '@highlight-ui/utils-hooks';
import { parseMetadata } from '@highlight-ui/utils-commons';
import classnames from 'classnames';
import toPairs from 'lodash/toPairs';
import groupBy from 'lodash/groupBy';
import noop from 'lodash/fp/noop';
import Icon from '@highlight-ui/icon';
import { Checkbox } from '@highlight-ui/checkbox';
import Typography from '@highlight-ui/typography';
import Avatar from '@highlight-ui/avatar';
import uniqueId from 'lodash/uniqueId';
import Button from '@highlight-ui/button';
import ColorPicker from '@highlight-ui/color-picker';
import { Input } from '@highlight-ui/input';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

function getSelectedValues(_a) {
    var multiple = _a.multiple, option = _a.option, selectedOptions = _a.selectedOptions, selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly;
    if (!multiple) {
        return [option];
    }
    if (selectedOptionsWithValuesOnly.includes(option.value)) {
        return selectedOptions.filter(function (selectedOption) { return selectedOption.value !== option.value; });
    }
    return __spreadArray(__spreadArray([], selectedOptions), [option]);
}
/**
 * Returns options that are pre-selected and disabled.
 * @param allOptions
 * @param preSelectedOptions
 */
function getFixedOptions(allOptions, preSelectedOptions) {
    var allDisabled = allOptions.filter(function (option) { return option.disabled; });
    return allDisabled.filter(function (disabledOption) {
        return preSelectedOptions.some(function (option) { return option.value === disabledOption.value; });
    });
}
/**
 * Returns a list of option that are fixed and are enabled
 * @param options
 * @param preSelectedOptions
 */
function getAllSelectableOptions(options, preSelectedOptions) {
    var allFixed = getFixedOptions(options, preSelectedOptions);
    var allEnabled = options.filter(function (option) { return !option.disabled; });
    return __spreadArray(__spreadArray([], allFixed), allEnabled);
}

var styles$c = {"label":"List-module__3eBtAqWo__v1-0-7","subLabel":"List-module__1Nm6OsGP__v1-0-7","labelWrapper":"List-module__2N1Czcx___v1-0-7","optionIcon":"List-module__20whLBmg__v1-0-7","option":"List-module__WP03aTyu__v1-0-7","optionCheckbox":"List-module__1xCWbakz__v1-0-7","multiLineOption":"List-module__2XiLBsmo__v1-0-7","selectedText":"List-module__3JZZHD7o__v1-0-7","disabled":"List-module__2wCQU5ux__v1-0-7","multiLineIcon":"List-module__1vZ64tez__v1-0-7","multiLineCheckbox":"List-module__37RkH873__v1-0-7","list":"List-module__2ObHKkci__v1-0-7","multiLineList":"List-module__1qnvYofW__v1-0-7","fullWidth":"List-module__3axRkzKW__v1-0-7","noResults":"List-module__n88tLk_9__v1-0-7"};

var styles$b = {"labelWrapper":"Option-module__1--7xK7L__v1-0-7","optionIcon":"Option-module__peJcMBH7__v1-0-7","option":"Option-module__2ym46zUu__v1-0-7","optionCheckbox":"Option-module__zaLdwsO3__v1-0-7","multiLineOption":"Option-module__1qplZ2cB__v1-0-7","selectedText":"Option-module__3WqFb6sf__v1-0-7","disabled":"Option-module__2dJafLVp__v1-0-7","multiLineIcon":"Option-module__1CYZ9njy__v1-0-7","multiLineCheckbox":"Option-module__2pPtBn6k__v1-0-7"};

var styles$a = {"highlighted":"TextWithHighlight-module__967BkGka__v1-0-7"};

// eslint-disable-next-line import/prefer-default-export
var searchSubstring = function (string, substring) {
    return string.toLowerCase().indexOf(substring.toLowerCase());
};

var TextWithHighlight = function (_a) {
    var highlightedText = _a.highlightedText, text = _a.text;
    var startIndex = searchSubstring(text, highlightedText);
    if (startIndex === -1)
        return React.createElement(React.Fragment, null, text);
    var endIndex = startIndex + highlightedText.length;
    return (React.createElement(React.Fragment, null,
        text.substring(0, startIndex),
        React.createElement("span", { className: styles$a.highlighted }, text.substring(startIndex, endIndex)),
        text.substring(endIndex)));
};

function SingleLineOptionLabel(_a) {
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label }));
}

var styles$9 = {"label":"MultiLineOptionLabel-module__3fgNK2Xm__v1-0-7","subLabel":"MultiLineOptionLabel-module__1iJeXNPV__v1-0-7","disabled":"MultiLineOptionLabel-module__2ftMukhR__v1-0-7"};

function MultiLineOptionLabel(_a) {
    var _b;
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(React.Fragment, null,
        React.createElement("span", { className: styles$9.label },
            React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })),
        React.createElement(Typography, { className: classnames(styles$9.subLabel, (_b = {},
                _b[styles$9.disabled] = option.disabled,
                _b)), component: "span", variant: "body-1", title: option.subLabel }, option.subLabel)));
}

var styles$8 = {"colorIcon":"ColoredOptionLabel-module__12fSpv1G__v1-0-7"};

function ColoredOptionLabel(_a) {
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(React.Fragment, null,
        React.createElement("span", { className: styles$8.colorIcon, style: { color: !option.disabled ? option.color : undefined } },
            React.createElement(Icon, { name: "circle", type: "solid", size: 12 })),
        React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })));
}

var styles$7 = {"label":"AvatarOptionLabel-module__3XTEg39V__v1-0-7","subLabel":"AvatarOptionLabel-module__SE9NqSW9__v1-0-7","wrapper":"AvatarOptionLabel-module__3MbElNjg__v1-0-7","avatar":"AvatarOptionLabel-module__duV8vrCr__v1-0-7","disabledAvatar":"AvatarOptionLabel-module__3H7pKms-__v1-0-7","labelsWrapper":"AvatarOptionLabel-module__1iX_lfeO__v1-0-7","disabled":"AvatarOptionLabel-module__BohvL_H4__v1-0-7"};

function AvatarOptionLabel(_a) {
    var _b, _c;
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement("div", { className: styles$7.wrapper },
        React.createElement(Avatar, { src: option.avatar, name: option.label, containerClassname: classnames(styles$7.avatar, (_b = {},
                _b[styles$7.disabledAvatar] = option.disabled,
                _b)) }),
        React.createElement("div", { className: styles$7.labelsWrapper },
            React.createElement("span", { className: styles$7.label },
                React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })),
            React.createElement(Typography, { className: classnames(styles$7.subLabel, (_c = {},
                    _c[styles$7.disabled] = option.disabled,
                    _c)), component: "span", variant: "body-1", title: option.subLabel }, option.subLabel))));
}

var optionLabelComponents = {
    'single-line': SingleLineOptionLabel,
    'multi-line': MultiLineOptionLabel,
    colored: ColoredOptionLabel,
    avatar: AvatarOptionLabel,
};
function Option(_a) {
    var _b, _c, _d;
    var disabled = _a.disabled, highlightedText = _a.highlightedText, onSelect = _a.onSelect, option = _a.option, optionStyle = _a.optionStyle, optionType = _a.optionType, selected = _a.selected;
    return (React.createElement("li", __assign({ "aria-selected": selected, className: classnames(styles$b.option, (_b = {},
            _b[styles$b.selectedText] = selected && optionStyle === 'checkmark',
            _b[styles$b.multiLineOption] = optionType === 'multi-line',
            _b[styles$b.disabled] = disabled,
            _b)), onClick: !disabled ? onSelect : noop, onKeyDown: !disabled ? onSelect : noop, role: "option", title: option.label, tabIndex: -1 }, parseMetadata(option.metadata)),
        optionStyle === 'checkbox' ? (React.createElement(Checkbox, { className: classnames(styles$b.optionCheckbox, (_c = {},
                _c[styles$b.multiLineCheckbox] = optionType === 'multi-line',
                _c)), checked: selected, readOnly: true, disabled: disabled })) : (React.createElement(Icon, { className: classnames(styles$b.optionIcon, (_d = {},
                _d[styles$b.multiLineIcon] = optionType === 'multi-line',
                _d)), name: "check", size: 14, type: "solid" })),
        React.createElement("div", { className: styles$b.labelWrapper }, React.createElement(optionLabelComponents[optionType], {
            option: option,
            highlightedText: highlightedText,
        }))));
}

var styles$6 = {"optionGroup":"OptionGroup-module__3LhwaP6p__v1-0-7","optionGroupName":"OptionGroup-module__2ZREs1EZ__v1-0-7"};

function OptionGroup(_a) {
    var name = _a.name, children = _a.children;
    return (React.createElement("li", { className: styles$6.optionGroup },
        React.createElement(Typography, { className: styles$6.optionGroupName, variant: "subtitle-2", component: "div" }, name),
        React.createElement("ul", null, children)));
}

var styles$5 = {"wrapper":"SearchInput-module__2ZvRInZD__v1-0-7","icon":"SearchInput-module__13jt8OmH__v1-0-7"};

var SearchInputWithRef = function (_a, ref) {
    var _b = _a.placeholder, placeholder = _b === void 0 ? '' : _b, value = _a.value, onTextChange = _a.onTextChange;
    var id = useState(function () { return uniqueId('component-'); })[0];
    var handleChange = function (event) {
        return onTextChange(event.currentTarget.value);
    };
    return (React.createElement("div", { className: styles$5.wrapper, role: "search" },
        React.createElement("label", { htmlFor: id },
            React.createElement(Icon, { className: styles$5.icon, color: "neutral-gray", name: "search", size: 12 }),
            React.createElement("input", { defaultValue: value, id: id, onChange: handleChange, placeholder: placeholder, ref: ref, type: "search" }))));
};
SearchInputWithRef.displayName = 'SearchInput';
var SearchInput = React.forwardRef(SearchInputWithRef);

var styles$4 = {"footerActionsContainer":"FooterActions-module__3ISOQf4m__v1-0-7","actionButton":"FooterActions-module__3KyK_euh__v1-0-7","light":"FooterActions-module__3BocMnWC__v1-0-7","dark":"FooterActions-module__1YQjdnE7__v1-0-7"};

function ActionButton(_a) {
    var className = _a.className, onClick = _a.onClick, text = _a.text;
    return (React.createElement(Typography, { component: "button", variant: "button-1", className: classnames(styles$4.actionButton, className), onClick: onClick, type: "button" }, text));
}
function FooterActions(_a) {
    var clearText = _a.clearText, onClear = _a.onClear, onSelectAll = _a.onSelectAll, selectAllText = _a.selectAllText;
    return (React.createElement("footer", { className: styles$4.footerActionsContainer },
        React.createElement(ActionButton, { className: styles$4.dark, onClick: onClear, text: clearText }),
        React.createElement(ActionButton, { className: styles$4.light, onClick: onSelectAll, text: selectAllText })));
}

var styles$3 = {"creatableFooterContainer":"AddOptionsFooter-module__3lpPIsIg__v1-0-7","inputWrapper":"AddOptionsFooter-module__2h7N5O7p__v1-0-7","colorPicker":"AddOptionsFooter-module__2tQh2gdt__v1-0-7"};

function AddOptionsFooter(_a) {
    var createButtonProps = _a.createButtonProps, inputProps = _a.inputProps, onCreate = _a.onCreate, colorPickerOptions = _a.colorPickerOptions;
    var _b = useState(''), inputValue = _b[0], setInputValue = _b[1];
    var _c = useState(colorPickerOptions === null || colorPickerOptions === void 0 ? void 0 : colorPickerOptions.initialColor), color = _c[0], setColor = _c[1];
    var onAddHandler = function () {
        onCreate(__assign({ value: inputValue || '' }, (color ? { color: color } : {})));
    };
    var onInputChange = function (event) {
        setInputValue(event.target.value);
    };
    return (React.createElement("footer", { className: styles$3.creatableFooterContainer },
        React.createElement(Input, { placeholder: inputProps === null || inputProps === void 0 ? void 0 : inputProps.placeholder, className: styles$3.inputWrapper, onKeyPress: function (e) {
                if (e.key.toLowerCase() === 'enter' && inputValue)
                    onAddHandler();
            }, metadata: inputProps === null || inputProps === void 0 ? void 0 : inputProps.metadata, onChange: onInputChange }),
        colorPickerOptions && (React.createElement("div", { className: styles$3.colorPicker },
            React.createElement(ColorPicker, { closeOnSelect: true, initialColor: color, metadata: colorPickerOptions.metadata, onChange: setColor, palette: colorPickerOptions.palette, title: colorPickerOptions.title }))),
        React.createElement(Button, { theme: "primary", onClick: onAddHandler, loading: createButtonProps.loading, disabled: createButtonProps.loading || !inputValue.length }, createButtonProps.label)));
}

var MIN_OPTIONS_TO_DISPLAY_SEARCH = 7.5;
var MIN_OPTIONS_TO_DISPLAY_ACTIONS = 7.5;
var MIN_GROUPS_TO_DISPLAY_HEADER = 2;
function List(_a, ref) {
    var _b;
    var shouldHideFooterActions = _a.shouldHideFooterActions, footerActionsClearText = _a.footerActionsClearText, footerActionsSelectAllText = _a.footerActionsSelectAllText, variant = _a.variant, multiple = _a.multiple, onClear = _a.onClear, onSelect = _a.onSelect, onSelectAll = _a.onSelectAll, onSearchChange = _a.onSearchChange, options = _a.options, optionType = _a.optionType, renderEmptyState = _a.renderEmptyState, searchable = _a.searchable, searchInitialValue = _a.searchInitialValue, searchAutofocus = _a.searchAutofocus, searchPlaceholder = _a.searchPlaceholder, searchNoResultsLabel = _a.searchNoResultsLabel, selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly, creatableOptions = _a.creatableOptions, style = _a.style, restProps = __rest(_a, ["shouldHideFooterActions", "footerActionsClearText", "footerActionsSelectAllText", "variant", "multiple", "onClear", "onSelect", "onSelectAll", "onSearchChange", "options", "optionType", "renderEmptyState", "searchable", "searchInitialValue", "searchAutofocus", "searchPlaceholder", "searchNoResultsLabel", "selectedOptionsWithValuesOnly", "creatableOptions", "style"]);
    var _c = useState(searchInitialValue), searchInputValue = _c[0], setSearchInputValue = _c[1];
    var searchInputRef = useRef(null);
    useEffect(function () {
        if (!searchAutofocus || !searchInputRef.current)
            return;
        searchInputRef.current.focus();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    var filteredOptions = options.filter(function (option) { return searchSubstring(option.label, searchInputValue) !== -1; });
    var groupedOptions = groupBy(filteredOptions, function (option) { return option.group; });
    var shouldShowNoResultsLabel = searchInputValue !== '' && filteredOptions.length === 0;
    var shouldShowActions = !shouldHideFooterActions &&
        multiple &&
        options.length > MIN_OPTIONS_TO_DISPLAY_ACTIONS &&
        !creatableOptions;
    var shouldShowSearchInput = searchable && options.length > MIN_OPTIONS_TO_DISPLAY_SEARCH;
    var shouldShowGroups = Object.keys(groupedOptions).length >= MIN_GROUPS_TO_DISPLAY_HEADER;
    var renderOptions = function (optionsList) {
        return optionsList.map(function (option) {
            var handleSelect = function () { return onSelect(option); };
            return (React.createElement(Option, { disabled: option.disabled, highlightedText: searchInputValue, key: option.value, onSelect: handleSelect, option: option, optionStyle: multiple ? 'checkbox' : 'checkmark', optionType: optionType, selected: selectedOptionsWithValuesOnly.includes(option.value) }));
        });
    };
    var renderOptionGroups = function () {
        return toPairs(groupedOptions).map(function (_a) {
            var groupName = _a[0], groupOptions = _a[1];
            return (React.createElement(OptionGroup, { key: groupName, name: groupName }, renderOptions(groupOptions)));
        });
    };
    var handleSearchInputChange = function (search) {
        setSearchInputValue(search);
        onSearchChange(search);
    };
    return (React.createElement("div", __assign({ ref: ref, className: classnames(styles$c.list, (_b = {},
            _b[styles$c.multiLineList] = optionType === 'multi-line',
            _b[styles$c.fullWidth] = variant === 'full-width',
            _b)), style: style }, restProps),
        shouldShowSearchInput && (React.createElement(SearchInput, { ref: searchInputRef, placeholder: searchPlaceholder, value: searchInputValue, onTextChange: handleSearchInputChange })),
        shouldShowNoResultsLabel && (React.createElement("div", { className: styles$c.noResults }, searchNoResultsLabel + " \"" + searchInputValue + "\"")),
        !shouldShowNoResultsLabel &&
            (options.length === 0 ? (renderEmptyState()) : (React.createElement("ul", { role: "listbox" }, shouldShowGroups
                ? renderOptionGroups()
                : renderOptions(filteredOptions)))),
        shouldShowActions && !shouldShowNoResultsLabel && (React.createElement(FooterActions, { clearText: footerActionsClearText, onClear: onClear, onSelectAll: onSelectAll, selectAllText: footerActionsSelectAllText })),
        creatableOptions && (React.createElement(AddOptionsFooter, { onCreate: creatableOptions.onCreate, createButtonProps: creatableOptions.createButtonProps, inputProps: creatableOptions.inputProps, colorPickerOptions: creatableOptions.colorPickerOptions }))));
}
// TODO: add strictly type checking with typescript [PPU-249]
var List$1 = forwardRef(List);

var styles$2 = {"defaultEmptyState":"DefaultEmptyState-module__2GMCdSkP__v1-0-7"};

function DefaultEmptyState(_a) {
    var _b = _a === void 0 ? { label: 'Nothing to display' } : _a, label = _b.label;
    return (React.createElement(Typography, { className: styles$2.defaultEmptyState, variant: "body-1", component: "div" }, label));
}

var styles$1 = {"selectTrigger":"DefaultTrigger-module__1Lcj4UJj__v1-0-7","selected":"DefaultTrigger-module__2pDPCVHO__v1-0-7","fullWidth":"DefaultTrigger-module__16yCD6zm__v1-0-7","innerText":"DefaultTrigger-module__1VgmRcUO__v1-0-7","outline-warning":"DefaultTrigger-module__3iRFUG-L__v1-0-7","outline-error":"DefaultTrigger-module__1zBxYiiH__v1-0-7","disabled":"DefaultTrigger-module__e9ZX9U5___v1-0-7"};

var styles = {"selectClearButton":"SelectClearButton-module__3m8ZA7Fl__v1-0-7","clearIcon":"SelectClearButton-module__3M_pe9ou__v1-0-7"};

var SelectClearButton = function (_a) {
    var onClick = _a.onClick, disabled = _a.disabled;
    return (React.createElement("div", { role: "button", className: styles.selectClearButton, onClick: disabled ? noop$1 : onClick, onKeyPress: disabled ? noop$1 : onClick, tabIndex: -1, "aria-label": "Clear Select Input" },
        React.createElement(Icon, { name: "times-circle", className: styles.clearIcon, type: "solid" })));
};

function DefaultTrigger(_a) {
    var _b;
    var disabled = _a.disabled, variant = _a.variant, label = _a.label, multiple = _a.multiple, onClick = _a.onClick, selectedOptions = _a.selectedOptions, visible = _a.visible, _c = _a.outline, outline = _c === void 0 ? 'default' : _c, triggerId = _a.triggerId, onClear = _a.onClear;
    var hasSelectedOption = selectedOptions.length > 0;
    var renderSingleSelectionLabel = function () {
        var firstItem = selectedOptions[0];
        return hasSelectedOption ? firstItem.label : label;
    };
    var renderMultiSelectionLabel = function () {
        return hasSelectedOption ? label + " \u00B7 " + selectedOptions.length : label;
    };
    var isFullWidth = variant === 'full-width';
    return (React.createElement("button", __assign({ className: classnames(styles$1.selectTrigger, styles$1["outline-" + outline], (_b = {},
            _b[styles$1.fullWidth] = isFullWidth,
            _b[styles$1.selected] = hasSelectedOption,
            _b[styles$1.disabled] = disabled,
            _b)), type: "button", disabled: disabled, onClick: onClick }, (triggerId ? { id: triggerId } : {})),
        React.createElement("span", { className: styles$1.innerText }, multiple ? renderMultiSelectionLabel() : renderSingleSelectionLabel()),
        onClear && selectedOptions.length > 0 ? (React.createElement(SelectClearButton, { onClick: onClear, disabled: disabled })) : null,
        React.createElement(Icon, { className: styles$1.icon, name: visible ? 'angle-up' : 'angle-down', type: "solid" })));
}

function SelectWithRef(_a, ref) {
    var className = _a.className, _b = _a.closeOnSelect, closeOnSelect = _b === void 0 ? false : _b, _c = _a.disabled, disabled = _c === void 0 ? false : _c, _d = _a.variant, variant = _d === void 0 ? 'inline' : _d, _e = _a.shouldHideFooterActions, shouldHideFooterActions = _e === void 0 ? false : _e, _f = _a.footerActionsClearText, footerActionsClearText = _f === void 0 ? 'Clear' : _f, _g = _a.footerActionsSelectAllText, footerActionsSelectAllText = _g === void 0 ? 'Select all' : _g, metadata = _a.metadata, _h = _a.multiple, multiple = _h === void 0 ? false : _h, _j = _a.onSearchChange, onSearchChange = _j === void 0 ? noop$1 : _j, _k = _a.onSelect, onSelect = _k === void 0 ? noop$1 : _k, options = _a.options, _l = _a.renderEmptyState, renderEmptyState = _l === void 0 ? DefaultEmptyState : _l, _m = _a.renderTrigger, renderTrigger = _m === void 0 ? DefaultTrigger : _m, _o = _a.optionType, optionType = _o === void 0 ? 'single-line' : _o, _p = _a.searchable, searchable = _p === void 0 ? true : _p, _q = _a.searchAutofocus, searchAutofocus = _q === void 0 ? true : _q, _r = _a.searchInitialValue, searchInitialValue = _r === void 0 ? '' : _r, _s = _a.searchNoResultsLabel, searchNoResultsLabel = _s === void 0 ? 'No results for' : _s, _t = _a.searchPlaceholder, searchPlaceholder = _t === void 0 ? 'Search' : _t, _u = _a.selectedOptions, selectedOptions = _u === void 0 ? [] : _u, _v = _a.triggerLabel, triggerLabel = _v === void 0 ? '' : _v, _w = _a.outline, outline = _w === void 0 ? 'default' : _w, triggerId = _a.triggerId, creatableOptions = _a.creatableOptions, _x = _a.isClearable, isClearable = _x === void 0 ? false : _x;
    var _y = useClickOutside(), clickOutsideRef = _y[0], hasClickedOutside = _y[1];
    var _z = useState(false), visible = _z[0], setVisibility = _z[1];
    var _0 = useAutoPositioner(), setReferenceElement = _0.setReferenceElement, setPopperElement = _0.setPopperElement, popperStyles = _0.styles.popper, attributes = _0.attributes;
    useEffect(function () { return setVisibility(false); }, [hasClickedOutside]);
    var selectedOptionsWithValuesOnly = useMemo(function () {
        return selectedOptions.map(function (selectedOption) { return selectedOption.value; });
    }, [selectedOptions]);
    var beforeSelect = function (callback) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (closeOnSelect)
                setVisibility(false);
            callback.apply(void 0, args);
        };
    };
    var handleClear = function (evt) {
        evt.stopPropagation();
        onSelect(getFixedOptions(options, selectedOptions));
    };
    var handleSelectAll = function () {
        return onSelect(getAllSelectableOptions(options, selectedOptions));
    };
    var handleSelect = function (option) {
        onSelect(getSelectedValues({
            option: option,
            selectedOptions: selectedOptions,
            selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly,
            multiple: multiple,
        }));
    };
    var handleTriggerClick = function () {
        return setVisibility(function (currentVisibility) { return !currentVisibility; });
    };
    var finalRef = useForkRef([clickOutsideRef, ref]);
    var _1 = useState(), triggerWidth = _1[0], setTriggerWidth = _1[1];
    var triggerRef = useForkRef([
        setReferenceElement,
        function (elem) {
            if (elem && variant === 'full-width')
                setTriggerWidth(getComputedStyle(elem).width);
        },
    ]);
    return (React.createElement("div", __assign({ className: className, ref: finalRef }, parseMetadata(metadata)),
        React.createElement("div", { ref: triggerRef }, renderTrigger({
            disabled: disabled,
            variant: variant,
            label: triggerLabel,
            multiple: multiple,
            onClick: handleTriggerClick,
            selectedOptions: selectedOptions,
            visible: visible,
            outline: outline,
            triggerId: triggerId,
            setListVisibility: setVisibility,
            onClear: isClearable ? handleClear : undefined,
        })),
        visible && (React.createElement(List$1, __assign({ shouldHideFooterActions: shouldHideFooterActions, footerActionsClearText: footerActionsClearText, footerActionsSelectAllText: footerActionsSelectAllText, variant: variant, multiple: multiple, onClear: beforeSelect(handleClear), onSelect: beforeSelect(handleSelect), onSelectAll: beforeSelect(handleSelectAll), onSearchChange: onSearchChange, options: options, optionType: optionType, renderEmptyState: renderEmptyState, selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly, searchable: searchable, searchNoResultsLabel: searchNoResultsLabel, searchPlaceholder: searchPlaceholder, searchInitialValue: searchInitialValue, searchAutofocus: searchAutofocus, creatableOptions: creatableOptions
                ? __assign(__assign({}, creatableOptions), { onCreate: function (createdItem) {
                        if (closeOnSelect && !multiple)
                            setVisibility(false);
                        creatableOptions.onCreate(createdItem);
                    } }) : creatableOptions, ref: setPopperElement, style: __assign(__assign({}, popperStyles), (variant === 'full-width' ? { width: triggerWidth } : {})) }, attributes.popper)))));
}
SelectWithRef.displayName = 'Select';
/**
 * According to this article: https://fettblog.eu/typescript-react-generic-forward-refs/
 */
var Select = React.forwardRef(SelectWithRef);

export default Select;
//# sourceMappingURL=index.js.map
